# -*- coding: utf-8 -*-

import numpy as np
import pytest
from peakdet.physio import Physio
from peakdet.tests import utils as testutils

DATA = np.loadtxt(testutils.get_test_data_path('ECG.csv'))
PROPERTIES = ['data', 'fs', 'history', 'peaks', 'troughs', '_masked']
PHYSIO_TESTS = [
    # accepts "correct" inputs for history
    dict(
        kwargs=dict(data=DATA, history=[('good', 'history')])
    ),
    # fails on bad inputs for history
    dict(
        kwargs=dict(data=DATA, history=['malformed', 'history']),
        raises=TypeError
    ),
    dict(
        kwargs=dict(data=DATA, history='not real history'),
        raises=TypeError
    ),
    # accepts "correct" for metadata
    dict(
        kwargs=dict(data=DATA, metadata=dict())
    ),
    dict(
        kwargs=dict(data=DATA, metadata=dict(peaks=[], reject=[], troughs=[]))
    ),
    # fails on bad inputs for metadata
    dict(
        kwargs=dict(data=DATA, metadata=[]),
        raises=TypeError
    ),
    dict(
        kwargs=dict(data=DATA, metadata=dict(peaks={})),
        raises=TypeError
    ),
    # fails on bad inputs for data
    dict(
        kwargs=dict(data=np.column_stack([DATA, DATA])),
        raises=ValueError
    ),
    dict(
        kwargs=dict(data='hello'),
        raises=ValueError
    )
]


def test_physio():
    phys = Physio(DATA, fs=1000)
    assert len(np.hstack((phys[:10], phys[10:-10], phys[-10:])))
    assert str(phys) == 'Physio(size=44611, fs=1000.0)'
    assert len(np.exp(phys)) == 44611


class TestPhysio():
    tests = PHYSIO_TESTS

    def test_physio_creation(self):
        for test in PHYSIO_TESTS:
            if test.get('raises') is not None:
                with pytest.raises(test['raises']):
                    phys = Physio(**test['kwargs'])
            else:
                phys = Physio(**test['kwargs'])
                for prop in PROPERTIES:
                    assert hasattr(phys, prop)
                for prop in ['peaks', 'reject', 'troughs']:
                    assert isinstance(phys._metadata.get(prop), np.ndarray)


def test_neurokit2phys(path_neurokit):
    df = pd.read_csv(path_neurokit, sep='\t')
    phys = Physio.neurokit2phys(path_neurokit, copy_data=True, copy_peaks=True, copy_troughs=True, fs=fs)

    assert all(np.unique(phys.data == np.hstack(df.loc[:, df.columns.str.endswith('Clean')].to_numpy())))
    assert all(np.unique(phys.peaks == np.where(df.loc[:, df.columns.str.endswith('Peaks')] != 0)[0]))
    assert phys.fs == fs


def test_phys2neurokit(path_phys):
    phys = load_physio(path_phys, allow_pickle=True)
    neuro = data.phys2neurokit(copy_data=True, copy_peaks=True, copy_troughs=False, module=module, neurokit_path=path_neurokit)

    assert all(np.unique(phys.data == np.hstack(neuro.loc[:, neuro.columns.str.endswith('Raw')].to_numpy())))
    assert all(np.unique(phys.peaks == np.where(neuro.loc[:, neuro.columns.str.endswith('Peaks')] != 0)[0]))



# -*- coding: utf-8 -*-
"""
Functions for interacting with physiological data acquired by external packages
"""

import warnings
import numpy as np
from peakdet import physio, utils


@utils.make_operation(exclude=[])
def load_rtpeaks(fname, channel, fs):
    """
    Loads data file as obtained from the ``rtpeaks`` Python module

    Data file `fname` should have a single, comma-delimited header of format:

        time,channel#,channel#,...,channel#

    Raw data should be stored in columnar format, also comma-delimited, beneath
    this header. All data should be stored as integers. For more information,
    see the ``rtpeaks`` homepage: https://github.com/rmarkello/rtpeaks.

    Parameters
    ----------
    fname : str
        Path to data file to be loaded
    channel : int
        Integer corresponding to the channel number in `fname` from which data
        should be loaded
    fs : float
        Sampling rate at which `fname` was acquired

    Returns
    -------
    data : :class:`peakdet.Physio`
        Loaded physiological data
    """

    if fname.startswith('/'):
        warnings.warn('Provided file seems to be an absolute path. In order '
                      'to ensure full reproducibility it is recommended that '
                      'a relative path is provided.')

    with open(fname, 'r') as src:
        header = src.readline().strip().split(',')

    col = header.index('channel{}'.format(channel))
    data = np.loadtxt(fname, usecols=col, skiprows=1, delimiter=',')
    phys = physio.Physio(data, fs=fs)

    return phys



# -*- coding: utf-8 -*-
"""
Helper class for holding physiological data and associated metadata inforamtion
"""

import numpy as np



class Physio():
    """
    Class to hold physiological data and relevant information

    Parameters
    ----------
    data : array_like
        Input data array
    fs : float, optional
        Sampling rate of `data` (Hz). Default: None
    history : list of tuples, optional
        Functions performed on `data`. Default: None
    metadata : dict, optional
        Metadata associated with `data`. Default: None
    suppdata : array_like, optional
        Support data array. Default: None

    Attributes
    ----------
    data : :obj:`numpy.ndarray`
        Physiological waveform
    fs : float
        Sampling rate of `data` in Hz
    history : list of tuples
        History of functions that have been performed on `data`, with relevant
        parameters provided to functions.
    peaks : :obj:`numpy.ndarray`
        Indices of peaks in `data`
    troughs : :obj:`numpy.ndarray`
        Indices of troughs in `data`
    suppdata : :obj:`numpy.ndarray`
        Secondary physiological waveform
    """

    def __init__(self, data, fs=None, history=None, metadata=None, suppdata=None):
        self._data = np.asarray(data).squeeze()
        if self.data.ndim > 1:
            raise ValueError('Provided data dimensionality {} > 1.'
                             .format(self.data.ndim))
        if not np.issubdtype(self.data.dtype, np.number):
            raise ValueError('Provided data of type {} is not numeric.'
                             .format(self.data.dtype))
        self._fs = np.float64(fs)
        self._history = [] if history is None else history
        if (not isinstance(self._history, list)
                or any([not isinstance(f, tuple) for f in self._history])):
            raise TypeError('Provided history {} must be a list-of-tuples. '
                            'Please check inputs.'.format(history))
        if metadata is not None:
            if not isinstance(metadata, dict):
                raise TypeError('Provided metadata {} must be dict-like.'
                                .format(metadata))
            for k in ['peaks', 'troughs', 'reject']:
                metadata.setdefault(k, np.empty(0, dtype=int))
                if not isinstance(metadata.get(k), np.ndarray):
                    try:
                        metadata[k] = np.asarray(metadata.get(k), dtype=int)
                    except TypeError:
                        raise TypeError('Provided metadata must be dict-like'
                                        'with integer array entries.')
            self._metadata = dict(**metadata)
        else:
            self._metadata = dict(peaks=np.empty(0, dtype=int),
                                  troughs=np.empty(0, dtype=int),
                                  reject=np.empty(0, dtype=int))
        self._suppdata = None if suppdata is None else np.asarray(suppdata).squeeze()

    def __array__(self):
        return self.data

    def __getitem__(self, slicer):
        return self.data[slicer]

    def __len__(self):
        return len(self.data)

    def __str__(self):
        return '{name}(size={size}, fs={fs})'.format(
            name=self.__class__.__name__,
            size=self.data.size,
            fs=self.fs
        )

    __repr__ = __str__

    @property
    def data(self):
        """ Physiological data """
        return self._data

    @property
    def fs(self):
        """ Sampling rate of data (Hz) """
        return self._fs

    @property
    def history(self):
        """ Functions that have been performed on / modified `data` """
        return self._history

    @property
    def peaks(self):
        """ Indices of detected peaks in `data` """
        return self._masked.compressed()

    @property
    def troughs(self):
        """ Indices of detected troughs in `data` """
        return self._metadata['troughs']

    @property
    def _masked(self):
        return np.ma.masked_array(self._metadata['peaks'],
                                  mask=np.isin(self._metadata['peaks'],
                                               self._metadata['reject']))

    @property
    def suppdata(self):
        """ Physiological data """
        return self._suppdata

    def phys2neurokit(self, copy_data, copy_peaks, copy_troughs, module, neurokit_path=None):
        """ Physio to neurokit dataframe

        Parameters
        ----------
        copy_data: bool
            whether to copy raw data from Physio object to dataframe
        copy_peaks: bool
            whether to copy peaks from Physio object to dataframe
        copy_troughs: bool
            whether to copy troughs from Physio object to dataframe
        module: string
            name of module (eg. 'EDA', 'RSP', 'PPG'...)
        neurokit_path: string
            path to neurokit dataframe
        """
        import pandas as pd

        if neurokit_path is not None:
            df = pd.read_csv(neurokit_path, sep='\t')
        else:
            df = pd.DataFrame(0, index=np.arange(len(self.data)), columns=['%s_Raw' % module, '%s_Peaks' % module, '%s_Troughs' % module])

        if copy_data:
            df.loc[:, df.columns.str.endswith('Raw')] = self.data

        if copy_peaks:
            b_peaks = np.zeros(len(self.data))
            b_peaks[self.peaks] = 1
            df.loc[:, df.columns.str.endswith('Peaks')] = b_peaks

        if copy_troughs:
            b_troughs = np.zeros(len(self.data))
            b_troughs[self.troughs] = 1
            df.loc[:, df.columns.str.endswith('Troughs')] = b_troughs

        return df

    @classmethod
    def neurokit2phys(cls, neurokit_path, fs, copy_data, copy_peaks, copy_troughs, **kwargs):
        """Neurokit dataframe to phys

        Parameters
        ----------
        neurokit_path: string
            path to neurokit dataframe
        fs: float
            sampling rate
        copy_data: bool
            whether to copy raw data from Physio object to dataframe
        copy_peaks: bool
            whether to copy peaks from Physio object to dataframe
        copy_troughs: bool
            whether to copy troughs from Physio object to dataframe
        suppdata: array_like, optional
            Support data array. Default: None
        """
        import pandas as pd

        df = pd.read_csv(neurokit_path, sep='\t')

        if copy_data:
            # if cleaned data exists, substitute 'data' with cleaned data, else use raw data
            if df.columns.str.endswith('Clean').any():
                data = np.hstack(df.loc[:, df.columns.str.endswith('Clean')].to_numpy())
            elif df.columns.str.endswith('Raw').any():
                data = np.hstack(df.loc[:, df.columns.str.endswith('Raw')].to_numpy())

        if copy_peaks:
            # if peaks exists
            if df.columns.str.endswith('Peaks').any():
                peaks = np.where(df.loc[:, df.columns.str.endswith('Peaks')] == 1)[0]

        if copy_troughs:
            # if troughs exists
            if df.columns.str.endswith('Troughs').any():
                troughs = np.where(df.loc[:, df.columns.str.endswith('Troughs')] == 1)[0]

        if 'peaks' in locals() and 'troughs' in locals():
            metadata = dict(peaks=peaks, troughs=troughs)
        elif 'peaks' in locals() and 'troughs' not in locals():
            metadata = dict(peaks=peaks)

        return cls(data, fs=fs, metadata=metadata, **kwargs)



# -*- coding: utf-8 -*-

import os
import json
import numpy as np
import pytest
from peakdet import io, operations, physio
from peakdet.tests.utils import get_test_data_path


def test_load_physio():
    # try loading pickle file (from io.save_physio)
    pckl = io.load_physio(get_test_data_path('ECG.phys'), allow_pickle=True)
    assert isinstance(pckl, physio.Physio)
    assert pckl.data.size == 44611
    assert pckl.fs == 1000.
    with pytest.warns(UserWarning):
        pckl = io.load_physio(get_test_data_path('ECG.phys'), fs=500.,
                              allow_pickle=True)
    assert pckl.fs == 500.

    # try loading CSV file
    csv = io.load_physio(get_test_data_path('ECG.csv'))
    assert isinstance(csv, physio.Physio)
    assert np.allclose(csv, pckl)
    assert np.isnan(csv.fs)
    assert csv.history[0][0] == 'load_physio'

    # try loading array
    with pytest.warns(UserWarning):
        arr = io.load_physio(np.loadtxt(get_test_data_path('ECG.csv')))
    assert isinstance(arr, physio.Physio)
    arr = io.load_physio(np.loadtxt(get_test_data_path('ECG.csv')),
                         history=[('np.loadtxt', {'fname': 'ECG.csv'})])
    assert isinstance(arr, physio.Physio)

    # try loading physio object (and resetting dtype)
    out = io.load_physio(arr, dtype='float32')
    assert out.data.dtype == np.dtype('float32')
    assert out.history[0][0] == 'np.loadtxt'
    assert out.history[-1][0] == 'load_physio'
    with pytest.raises(TypeError):
        io.load_physio([1, 2, 3])


def test_save_physio(tmpdir):
    pckl = io.load_physio(get_test_data_path('ECG.phys'), allow_pickle=True)
    out = io.save_physio(tmpdir.join('tmp').purebasename, pckl)
    assert os.path.exists(out)
    assert isinstance(io.load_physio(out, allow_pickle=True), physio.Physio)


def test_load_history(tmpdir):
    # get paths of data, new history
    fname = get_test_data_path('ECG.csv')
    temp_history = tmpdir.join('tmp').purebasename

    # make physio object and perform some operations
    phys = io.load_physio(fname, fs=1000.)
    filt = operations.filter_physio(phys, [5., 15.], 'bandpass')

    # save history to file and recreate new object from history
    path = io.save_history(temp_history, filt)
    replayed = io.load_history(path, verbose=True)

    # ensure objects are the same
    assert np.allclose(filt, replayed)
    assert filt.history == replayed.history
    assert filt.fs == replayed.fs


def test_save_history(tmpdir):
    # get paths of data, original history, new history
    fname = get_test_data_path('ECG.csv')
    orig_history = get_test_data_path('history.json')
    temp_history = tmpdir.join('tmp').purebasename

    # make physio object and perform some operations
    phys = physio.Physio(np.loadtxt(fname), fs=1000.)
    with pytest.warns(UserWarning):  # no history = warning
        io.save_history(temp_history, phys)
    filt = operations.filter_physio(phys, [5., 15.], 'bandpass')
    path = io.save_history(temp_history, filt)  # dump history=

    # load both original and new json and ensure equality
    with open(path, 'r') as src:
        hist = json.load(src)
    with open(orig_history, 'r') as src:
        orig = json.load(src)
    assert hist == orig



# -*- coding: utf-8 -*-

import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import pytest
from peakdet import operations
from peakdet.physio import Physio
from peakdet.tests import utils as testutils

data = np.loadtxt(testutils.get_test_data_path('ECG.csv'))
WITHFS = Physio(data, fs=1000.)
NOFS = Physio(data)


def test_filter_physio():
    # check lowpass and highpass filters
    for meth in ['lowpass', 'highpass']:
        params = dict(cutoffs=2, method=meth)
        assert len(WITHFS) == len(operations.filter_physio(WITHFS, **params))
        params['order'] = 5
        assert len(WITHFS) == len(operations.filter_physio(WITHFS, **params))
        params['cutoffs'] = [2, 10]
        with pytest.raises(ValueError):
            operations.filter_physio(WITHFS, **params)
        with pytest.raises(ValueError):
            operations.filter_physio(NOFS, **params)
    # check bandpass and bandstop filters
    for meth in ['bandpass', 'bandstop']:
        params = dict(cutoffs=[2, 10], method=meth)
        assert len(WITHFS) == len(operations.filter_physio(WITHFS, **params))
        params['order'] = 5
        assert len(WITHFS) == len(operations.filter_physio(WITHFS, **params))
        params['cutoffs'] = 2
        with pytest.raises(ValueError):
            operations.filter_physio(WITHFS, **params)
        with pytest.raises(ValueError):
            operations.filter_physio(NOFS, **params)
    # check appropriate filter methods
    with pytest.raises(ValueError):
        operations.filter_physio(WITHFS, 2, 'notafilter')
    # check nyquist
    with pytest.raises(ValueError):
        operations.filter_physio(WITHFS, [2, 1000], 'bandpass')


def test_interpolate_physio():
    with pytest.raises(ValueError):
        operations.interpolate_physio(NOFS, 100.)
    for fn in [50, 100, 200, 500, 2000, 5000]:
        new = operations.interpolate_physio(WITHFS, fn)
        assert new.fs == fn
        if fn < WITHFS.fs:
            assert len(new) < len(WITHFS)
        else:
            assert len(new) > len(WITHFS)


def test_peakfind_physio():
    with pytest.raises(ValueError):
        operations.peakfind_physio(NOFS)
    operations.peakfind_physio(NOFS, dist=20)
    operations.peakfind_physio(NOFS, thresh=0.4, dist=20)
    operations.peakfind_physio(WITHFS)
    operations.peakfind_physio(WITHFS, dist=20)
    operations.peakfind_physio(WITHFS, thresh=0.4)


def test_delete_peaks():
    to_delete = [24685, 44169]
    peaks = operations.peakfind_physio(WITHFS)
    deleted = operations.delete_peaks(peaks, to_delete)
    assert len(deleted.peaks) == len(peaks.peaks) - len(to_delete)


def test_reject_peaks():
    to_reject = [24685, 44169]
    peaks = operations.peakfind_physio(WITHFS)
    rejected = operations.reject_peaks(peaks, to_reject)
    assert len(rejected.peaks) == len(peaks.peaks) - len(to_reject)


def test_edit_physio():
    # value error when no sampling rate provided for interactive editing
    with pytest.raises(ValueError):
        operations.edit_physio(NOFS)
    # if sampling rate provided but no peaks/troughs just return
    operations.edit_physio(WITHFS)


def test_plot_physio():
    for data in [NOFS, WITHFS]:
        assert isinstance(operations.plot_physio(data), matplotlib.axes.Axes)
    peaks = operations.peakfind_physio(WITHFS)
    assert isinstance(operations.plot_physio(peaks), matplotlib.axes.Axes)
    fig, ax = plt.subplots(1, 1)
    assert ax == operations.plot_physio(peaks, ax=ax)



# -*- coding: utf-8 -*-
import glob
import os
import sys
import warnings
import matplotlib
matplotlib.use('WXAgg')
from gooey import Gooey, GooeyParser
import peakdet

TARGET = 'pythonw' if sys.platform == 'darwin' else 'python'
TARGET += ' -u ' + os.path.abspath(__file__)

LOADERS = dict(
    rtpeaks=peakdet.load_rtpeaks,
    MRI=peakdet.load_physio
)

MODALITIES = dict(
    ECG=([5., 15.], 'bandpass'),
    PPG=(2, 'lowpass'),
    RESP=([0.05, 0.5], 'bandpass')
)

ATTR_CONV = {
    'Average NN intervals': 'avgnn',
    'Root mean square of successive differences': 'rmssd',
    'Standard deviation of NN intervals': 'sdnn',
    'Standard deviation of successive differences': 'sdsd',
    'Number of successive differences >50 ms': 'nn50',
    'Percent of successive differences >50 ms': 'pnn50',
    'Number of successive differences >20 ms': 'nn20',
    'Percent of successive differences >20 ms': 'pnn20',
    'High frequency HRV hfHRV': 'hf',
    'Log of high frequency HRV, log(hfHRV)': 'hf_log',
    'Low frequency HRV, lfHRV': 'lf',
    'Log of low frequency HRV, log(lfHRV)': 'lf_log',
    'Very low frequency HRV, vlfHRV': 'vlf',
    'Log of very low frequency HRV, log(vlfHRV)': 'vlf_log',
    'Ratio of lfHRV : hfHRV': 'lftohf',
    'Peak frequency of hfHRV': 'hf_peak',
    'Peak frequency of lfHRV': 'lf_peak'
}


@Gooey(program_name='Physio pipeline',
       program_description='Physiological processing pipeline',
       default_size=(800, 600),
       target=TARGET)
def get_parser():
    """ Parser for GUI and command-line arguments """
    parser = GooeyParser()
    parser.add_argument('file_template', metavar='Filename template',
                        widget='FileChooser',
                        help='Select a representative file and replace all '
                             'subject-specific information with a "?" symbol.'
                             '\nFor example, subject_001_data.txt should '
                             'become subject_???_data.txt and will expand to '
                             'match\nsubject_001_data.txt, subject_002_data.'
                             'txt, ..., subject_999_data.txt.')

    inp_group = parser.add_argument_group('Inputs', 'Options to specify '
                                          'format of input files')
    inp_group.add_argument('--modality', metavar='Modality', default='ECG',
                           choices=list(MODALITIES.keys()),
                           help='Modality of input data.')
    inp_group.add_argument('--fs', metavar='Sampling rate', default=1000.0,
                           type=float,
                           help='Sampling rate of input data.')
    inp_group.add_argument('--source', metavar='Source', default='rtpeaks',
                           choices=list(LOADERS.keys()),
                           help='Program used to collect the data.')
    inp_group.add_argument('--channel', metavar='Channel', default=1, type=int,
                           help='Which channel of data to read from data '
                                'files.\nOnly applies if "Source" is set to '
                                'rtpeaks.')

    out_group = parser.add_argument_group('Outputs', 'Options to specify '
                                          'format of output files')
    out_group.add_argument('-o', '--output', metavar='Filename',
                           default='peakdet.csv',
                           help='Output filename for generated measurements.')
    out_group.add_argument('-m', '--measurements', metavar='Measurements',
                           nargs='+', widget='Listbox',
                           choices=list(ATTR_CONV.keys()),
                           default=['Average NN intervals',
                                    'Standard deviation of NN intervals'],
                           help='Desired physiological measurements.\nChoose '
                                'multiple with shift+click or ctrl+click.')
    out_group.add_argument('-s', '--savehistory', metavar='Save history',
                           action='store_true',
                           help='Whether to save history of data processing '
                                'for each file.')

    edit_group = parser.add_argument_group('Workflow arguments (optional!)',
                                           'Options to specify modifications '
                                           'to workflow')
    edit_group.add_argument('-n', '--noedit', metavar='Editing',
                            action='store_true',
                            help='Turn off interactive editing.')
    edit_group.add_argument('-t', '--thresh', metavar='Threshold', default=0.2,
                            type=float,
                            help='Threshold for peak detection algorithm.')

    return parser


def workflow(*, file_template, modality, fs, source='MRI', channel=1,
             output='peakdet.csv', savehistory=True, noedit=False, thresh=0.2,
             measurements=ATTR_CONV.keys()):
    """
    Basic workflow for physiological data

    Parameters
    ----------
    file_template : str
        Template filename for data inputs
    modality : {'ECG', 'PPG', 'RESP'}
        Currently support data modalities
    fs : float
        Sampling rate of input data
    source : {'rtpeaks', 'MRI'}, optional
        How data were acquired. Default: 'MRI'
    channel : int, optional
        Which channel of data to analyze; only applies if source is 'rtpeaks'.
        Default: 1
    output : str, optional
        Desired output filename. Default: 'peakdet.csv'
    savehistory : bool, optional
        Whether to save editing history of each file with
        ``peakdet.save_history``. History will be used if this workflow is
        run again on the samed data files. Default: True
    noedit : bool, optional
        Whether to disable interactive editing of physio data. Default: False
    thresh : [0, 1] float, optional
        Threshold for peak detection. Default: 0.2
    measurements : list, optional
        Which HRV-related measurements to save from data. See ``peakdet.HRV``
        for available measurements. Default: all available measurements.
    """

    # output file
    print('OUTPUT FILE:\t\t{}\n'.format(output))
    # grab files from file template
    print('FILE TEMPLATE:\t{}\n'.format(file_template))
    files = glob.glob(file_template, recursive=True)

    # convert measurements to peakdet.HRV attribute friendly names
    try:
        print('REQUESTED MEASUREMENTS: {}\n'.format(', '.join(measurements)))
    except TypeError:
        raise TypeError('It looks like you didn\'t select any of the options '
                        'specifying desired output measurements. Please '
                        'select at least one measurement and try again.')
    measurements = [ATTR_CONV[attr] for attr in measurements]

    # get appropriate loader
    load_func = LOADERS[source]

    # check if output file exists -- if so, ensure headers will match
    head = 'filename,' + ','.join(measurements)
    if os.path.exists(output):
        with open(output, 'r') as src:
            eheader = src.readlines()[0]
        # if existing output file does not have same measurements are those
        # requested on command line, warn and use existing measurements so
        # as not to totally fork up existing file
        if eheader != head:
            warnings.warn('Desired output file already exists and requested '
                          'measurements do not match with measurements in '
                          'existing output file. Using the pre-existing '
                          'measurements, instead.')
            measurements = [f.strip() for f in eheader.split(',')[1:]]
        head = ''
    # if output file doesn't exist, nbd
    else:
        head += '\n'

    with open(output, 'a+') as dest:
        dest.write(head)
        # iterate through all files and do peak detection with manual editing
        for fname in files:
            fname = os.path.relpath(fname)
            print('Currently processing {}'.format(fname))

            # if we want to save history, this is the output name it would take
            outname = os.path.join(os.path.dirname(fname),
                                   '.' + os.path.basename(fname) + '.json')

            # let's check if history already exists and load that file, if so
            if os.path.exists(outname):
                data = peakdet.load_history(outname)
            else:
                # load data with appropriate function, depending on source
                if source == 'rtpeaks':
                    data = load_func(fname, fs=fs, channel=channel)
                else:
                    data = load_func(fname, fs=fs)

                # filter
                flims, method = MODALITIES[modality]
                data = peakdet.filter_physio(data, cutoffs=flims,
                                             method=method)

                # perform peak detection
                data = peakdet.peakfind_physio(data, thresh=thresh)

            # edit peaks, if desired (HIGHLY RECOMMENDED)
            # we'll do this even if we loaded from history
            # just to give another chance to check things over
            if not noedit:
                data = peakdet.edit_physio(data)

            # save back out to history, if desired
            if savehistory:
                peakdet.save_history(outname, data)

            # keep requested outputs
            hrv = peakdet.HRV(data)
            outputs = ['{:.5f}'.format(getattr(hrv, attr, ''))
                       for attr in measurements]

            # save as we go so that interruptions don't screw everything up
            dest.write(','.join([fname] + outputs) + '\n')


def main():
    opts = get_parser().parse_args()
    workflow(**vars(opts))


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
Functions for processing and interpreting physiological data
"""

import matplotlib.pyplot as plt
import numpy as np
from scipy import interpolate, signal
from peakdet import editor, utils


@utils.make_operation()
def filter_physio(data, cutoffs, method, *, order=3):
    """
    Applies an `order`-order digital `method` Butterworth filter to `data`

    Parameters
    ----------
    data : Physio_like
        Input physiological data to be filtered
    cutoffs : int or list
        If `method` is 'lowpass' or 'highpass', an integer specifying the lower
        or upper bound of the filter (in Hz). If method is 'bandpass' or
        'bandstop', a list specifying the lower and upper bound of the filter
        (in Hz).
    method : {'lowpass', 'highpass', 'bandpass', 'bandstop'}
        The type of filter to apply to `data`
    order : int, optional
        Order of filter to be applied. Default: 3

    Returns
    -------
    filtered : :class:`peakdet.Physio`
        Filtered input `data`
    """

    _valid_methods = ['lowpass', 'highpass', 'bandpass', 'bandstop']

    data = utils.check_physio(data, ensure_fs=True)
    if method not in _valid_methods:
        raise ValueError('Provided method {} is not permitted; must be in {}.'
                         .format(method, _valid_methods))

    cutoffs = np.array(cutoffs)
    if method in ['lowpass', 'highpass'] and cutoffs.size != 1:
        raise ValueError('Cutoffs must be length 1 when using {} filter'
                         .format(method))
    elif method in ['bandpass', 'bandstop'] and cutoffs.size != 2:
        raise ValueError('Cutoffs must be length 2 when using {} filter'
                         .format(method))

    nyq_cutoff = cutoffs / (data.fs * 0.5)
    if np.any(nyq_cutoff > 1):
        raise ValueError('Provided cutoffs {} are outside of the Nyquist '
                         'frequency for input data with sampling rate {}.'
                         .format(cutoffs, data.fs))

    b, a = signal.butter(int(order), nyq_cutoff, btype=method)
    filtered = utils.new_physio_like(data, signal.filtfilt(b, a, data))

    return filtered


@utils.make_operation()
def interpolate_physio(data, target_fs, *, kind='cubic'):
    """
    Interpolates `data` to desired sampling rate `target_fs`

    Parameters
    ----------
    data : Physio_like
        Input physiological data to be interpolated
    target_fs : float
        Desired sampling rate for `data`
    kind : str or int, optional
        Type of interpolation to perform. Must be one of available kinds in
        :func:`scipy.interpolate.interp1d`. Default: 'cubic'

    Returns
    -------
    interp : :class:`peakdet.Physio`
        Interpolated input `data`
    """

    data = utils.check_physio(data, ensure_fs=True)

    factor = target_fs / data.fs

    # generate original and target "time" series
    t_orig = np.linspace(0, len(data) / data.fs, len(data))
    t_new = np.linspace(0, len(data) / data.fs, int(len(data) * factor))

    # interpolate data and generate new Physio object
    interp = interpolate.interp1d(t_orig, data, kind=kind)(t_new)
    if data.suppdata is None:
        suppinterp = None
    else:
        suppinterp = interpolate.interp1d(t_orig, data.suppdata, kind=kind)(t_new)
    interp = utils.new_physio_like(data, interp, fs=target_fs, suppdata=suppinterp)

    return interp


@utils.make_operation()
def peakfind_physio(data, *, thresh=0.2, dist=None):
    """
    Performs peak and trough detection on `data`

    Parameters
    ----------
    data : Physio_like
        Input data in which to find peaks
    thresh : float [0,1], optional
        Relative height threshold a data point must surpass to be classified as
        a peak. Default: 0.2
    dist : int, optional
        Distance in indices that peaks must be separated by in `data`. If None,
        this is estimated. Default: None

    Returns
    -------
    peaks : :class:`peakdet.Physio`
        Input `data` with detected peaks and troughs
    """

    ensure_fs = True if dist is None else False
    data = utils.check_physio(data, ensure_fs=ensure_fs, copy=True)

    # first pass peak detection to get approximate distance between peaks
    cdist = data.fs // 4 if dist is None else dist
    thresh = np.squeeze(np.diff(np.percentile(data, [5, 95]))) * thresh
    locs, heights = signal.find_peaks(data[:], distance=cdist, height=thresh)

    # second, more thorough peak detection
    cdist = np.diff(locs).mean() // 2
    heights = np.percentile(heights['peak_heights'], 1)
    locs, heights = signal.find_peaks(data[:], distance=cdist, height=heights)
    data._metadata['peaks'] = locs
    # perform trough detection based on detected peaks
    data._metadata['troughs'] = utils.check_troughs(data, data.peaks)

    return data


@utils.make_operation()
def delete_peaks(data, remove):
    """
    Deletes peaks in `remove` from peaks stored in `data`

    Parameters
    ----------
    data : Physio_like
    remove : array_like

    Returns
    -------
    data : Physio_like
    """

    data = utils.check_physio(data, ensure_fs=False, copy=True)
    data._metadata['peaks'] = np.setdiff1d(data._metadata['peaks'], remove)
    data._metadata['troughs'] = utils.check_troughs(data, data.peaks, data.troughs)

    return data


@utils.make_operation()
def reject_peaks(data, remove):
    """
    Marks peaks in `remove` as rejected artifacts in `data`

    Parameters
    ----------
    data : Physio_like
    remove : array_like

    Returns
    -------
    data : Physio_like
    """

    data = utils.check_physio(data, ensure_fs=False, copy=True)
    data._metadata['reject'] = np.append(data._metadata['reject'], remove)
    data._metadata['troughs'] = utils.check_troughs(data, data.peaks, data.troughs)

    return data


@utils.make_operation()
def add_peaks(data, add):
    """
    Add `newpeak` to add them in `data`

    Parameters
    ----------
    data : Physio_like
    add : int

    Returns
    -------
    data : Physio_like
    """

    data = utils.check_physio(data, ensure_fs=False, copy=True)
    idx = np.searchsorted(data._metadata['peaks'], add)
    data._metadata['peaks'] = np.insert(data._metadata['peaks'], idx, add)
    data._metadata['troughs'] = utils.check_troughs(data, data.peaks)

    return data


@utils.make_operation()
def add_peaks(data, add):
    """
    Add `newpeak` to add them in `data`

    Parameters
    ----------
    data : Physio_like
    add : int

    Returns
    -------
    data : Physio_like
    """

    data = utils.check_physio(data, ensure_fs=False, copy=True)
    idx = np.searchsorted(data._metadata['peaks'], add)
    data._metadata['peaks'] = np.insert(data._metadata['peaks'], idx, add)
    data._metadata['troughs'] = utils.check_troughs(data, data.peaks)

    return data


def edit_physio(data):
    """
    Opens interactive plot with `data` to permit manual editing of time series

    Parameters
    ----------
    data : Physio_like
        Physiological data to be edited

    Returns
    -------
    edited : :class:`peakdet.Physio`
        Input `data` with manual edits
    """

    data = utils.check_physio(data, ensure_fs=True)

    # no point in manual edits if peaks/troughs aren't defined
    if not (len(data.peaks) and len(data.troughs)):
        return

    # perform manual editing
    edits = editor._PhysioEditor(data)
    plt.show(block=True)

    # replay editing on original provided data object
    if len(edits.rejected) > 0:
        data = reject_peaks(data, remove=sorted(edits.rejected))
    if len(edits.deleted) > 0:
        data = delete_peaks(data, remove=sorted(edits.deleted))
    if len(edits.included) > 0:
        data = add_peaks(data, add=sorted(edits.included))

    return data


def plot_physio(data, *, ax=None):
    """
    Plots `data` and associated peaks / troughs

    Parameters
    ----------
    data : Physio_like
        Physiological data to plot
    ax : :class:`matplotlib.axes.Axes`, optional
        Axis on which to plot `data`. If None, a new axis is created. Default:
        None

    Returns
    -------
    ax : :class:`matplotlib.axes.Axes`
        Axis with plotted `data`
    """

    # generate x-axis time series
    fs = 1 if np.isnan(data.fs) else data.fs
    time = np.arange(0, len(data) / fs, 1 / fs)
    if ax is None:
        fig, ax = plt.subplots(1, 1)
    # plot data with peaks + troughs, as appropriate
    ax.plot(time, data, 'b',
            time[data.peaks], data[data.peaks], '.r',
            time[data.troughs], data[data.troughs], '.g')

    return ax



# -*- coding: utf-8 -*-
"""
Functions for loading and saving data and analyses
"""

import json
import os.path as op
import warnings
import numpy as np
from peakdet import physio, utils

EXPECTED = ['data', 'fs', 'history', 'metadata']


def load_physio(data, *, fs=None, dtype=None, history=None,
                allow_pickle=False):
    """
    Returns `Physio` object with provided data

    Parameters
    ----------
    data : str or array_like or Physio_like
        Input physiological data. If array_like, should be one-dimensional
    fs : float, optional
        Sampling rate of `data`. Default: None
    dtype : data_type, optional
        Data type to convert `data` to, if conversion needed. Default: None
    history : list of tuples, optional
        Functions that have been performed on `data`. Default: None
    allow_pickle : bool, optional
        Whether to allow loading if `data` contains pickled objects. Default:
        False

    Returns
    -------
    data: :class:`peakdet.Physio`
        Loaded physiological data

    Raises
    ------
    TypeError
        If provided `data` is unable to be loaded
    """

    # first check if the file was made with `save_physio`; otherwise, try to
    # load it as a plain text file and instantiate a history
    if isinstance(data, str):
        try:
            inp = dict(np.load(data, allow_pickle=allow_pickle))
            for attr in EXPECTED:
                try:
                    inp[attr] = inp[attr].dtype.type(inp[attr])
                except KeyError:
                    raise ValueError('Provided npz file {} must have all of '
                                     'the following attributes: {}'
                                     .format(data, EXPECTED))
            # fix history, which needs to be list-of-tuple
            if inp['history'] is not None:
                inp['history'] = list(map(tuple, inp['history']))
        except (IOError, OSError, ValueError):
            inp = dict(data=np.loadtxt(data),
                       history=[utils._get_call(exclude=[])])
        phys = physio.Physio(**inp)
    # if we got a numpy array, load that into a Physio object
    elif isinstance(data, np.ndarray):
        if history is None:
            warnings.warn('Loading data from a numpy array without providing a'
                          'history will render reproducibility functions '
                          'useless! Continuing anyways.')
        phys = physio.Physio(np.asarray(data, dtype=dtype), fs=fs,
                             history=history)
    # create a new Physio object out of a provided Physio object
    elif isinstance(data, physio.Physio):
        phys = utils.new_physio_like(data, data.data, fs=fs, dtype=dtype)
        phys._history += [utils._get_call()]
    else:
        raise TypeError('Cannot load data of type {}'.format(type(data)))

    # reset sampling rate, as requested
    if fs is not None and fs != phys.fs:
        if not np.isnan(phys.fs):
            warnings.warn('Provided sampling rate does not match loaded rate. '
                          'Resetting loaded sampling rate {} to provided {}'
                          .format(phys.fs, fs))
        phys._fs = fs
    # coerce datatype, if needed
    if dtype is not None:
        phys._data = np.asarray(phys[:], dtype=dtype)

    return phys


def save_physio(fname, data):
    """
    Saves `data` to `fname`

    Parameters
    ----------
    fname : str
        Path to output file; .phys will be appended if necessary
    data : Physio_like
        Data to be saved to file

    Returns
    -------
    fname : str
        Full filepath to saved output
    """

    from peakdet.utils import check_physio

    data = check_physio(data)
    fname += '.phys' if not fname.endswith('.phys') else ''
    with open(fname, 'wb') as dest:
        hist = data.history if data.history != [] else None
        np.savez_compressed(dest, data=data.data, fs=data.fs,
                            history=hist, metadata=data._metadata)

    return fname


def load_history(file, verbose=False):
    """
    Loads history from `file` and replays it, creating new Physio instance

    Parameters
    ----------
    file : str
        Path to input JSON file
    verbose : bool, optional
        Whether to print messages as history is being replayed. Default: False

    Returns
    -------
    file : str
        Full filepath to saved output
    """

    # import inside function for safety!
    # we'll likely be replaying some functions from within this module...
    import peakdet

    # grab history from provided JSON file
    with open(file, 'r') as src:
        history = json.load(src)

    # replay history from beginning and return resultant Physio object
    data = None
    for (func, kwargs) in history:
        if verbose:
            print('Rerunning {}'.format(func))
        # loading functions don't have `data` input because it should be the
        # first thing in `history` (when the data was originally loaded!).
        # for safety, check if `data` is None; someone could have potentially
        # called load_physio on a Physio object (which is a valid, albeit
        # confusing, thing to do)
        if 'load' in func and data is None:
            if not op.exists(kwargs['data']):
                if kwargs['data'].startswith('/'):
                    msg = ('Perhaps you are trying to load a history file '
                           'that was generated with an absolute path?')
                else:
                    msg = ('Perhaps you are trying to load a history file '
                           'that was generated from a different directory?')
                raise FileNotFoundError('{} does not exist. {}'
                                        .format(kwargs['data'], msg))
            data = getattr(peakdet, func)(**kwargs)
        else:
            data = getattr(peakdet, func)(data, **kwargs)

    return data


def save_history(file, data):
    """
    Saves history of physiological `data` to `file`

    Saved file can be replayed with `peakdet.load_history`

    Parameters
    ----------
    file : str
        Path to output file; .json will be appended if necessary
    data : Physio_like
        Data with history to be saved to file

    Returns
    -------
    file : str
        Full filepath to saved output
    """

    from peakdet.utils import check_physio

    data = check_physio(data)
    if len(data.history) == 0:
        warnings.warn('History of provided Physio object is empty. Saving '
                      'anyway, but reloading this file will result in an '
                      'error.')
    file += '.json' if not file.endswith('.json') else ''
    with open(file, 'w') as dest:
        json.dump(data.history, dest, indent=4)

    return file



# -*- coding: utf-8 -*-

from collections import namedtuple

import pytest

from peakdet import editor
from peakdet.tests.utils import get_peak_data


wheel = namedtuple('wheel', ('step'))
key = namedtuple('key', ('key',))


def test_PhysioEditor():
    edits = editor._PhysioEditor(get_peak_data())

    # test scroll functionality
    edits.on_wheel(wheel(10))

    # test reject / delete functionality
    for m in range(2):
        edits.on_remove(0, 10, reject=True)
        edits.on_remove(10, 20, reject=False)

    # undo delete + reject
    for m in range(2):
        edits.undo()

    # test key undo (and undo when history doesn't exist)
    edits.on_key(key('ctrl+z'))

    # redo so that there is history on quit
    edits.on_remove(0, 10, reject=True)
    edits.on_remove(10, 20, reject=False)

    # quit editor and clean up edits
    edits.on_key(key('ctrl+z'))

    with pytest.raises(TypeError):
        editor._PhysioEditor([0, 1, 2])



"""
Utilities for testing
"""

from os.path import join as pjoin
from pkg_resources import resource_filename
import numpy as np
from peakdet import io, operations
from peakdet.utils import _get_call


def get_call_func(arg1, arg2, *, kwarg1=10, kwarg2=20,
                  exclude=['exclude', 'serializable'],
                  serializable=True):
    """ Function for testing `peakdet.utils._get_call()` """
    if arg1 > 10:
        kwarg1 = kwarg1 + arg1
    if arg2 > 20:
        kwarg2 = kwarg2 + arg2
    return _get_call(exclude=exclude, serializable=serializable)


def get_test_data_path(fname=None):
    """ Function for getting `peakdet` test data path """
    path = resource_filename('peakdet', 'tests/data')
    return pjoin(path, fname) if fname is not None else path


def get_sample_data():
    """ Function for generating tiny sine wave form for testing """
    data = np.sin(np.linspace(0, 20, 40))
    peaks, troughs = np.array([3, 15, 28]), np.array([9, 21, 34])

    return data, peaks, troughs


def get_peak_data():
    """ Function for getting some pregenerated physio data """
    physio = io.load_physio(get_test_data_path('ECG.csv'), fs=1000)
    filt = operations.filter_physio(physio, [5., 15.], 'bandpass')
    peaks = operations.peakfind_physio(filt)

    return peaks



__all__ = [
    'delete_peaks', 'edit_physio', 'filter_physio', 'interpolate_physio',
    'peakfind_physio', 'plot_physio', 'reject_peaks',
    'load_physio', 'save_physio', 'load_history', 'save_history',
    'load_rtpeaks', 'Physio', 'HRV', '__version__'
]

from peakdet.analytics import (HRV)
from peakdet.external import (load_rtpeaks)
from peakdet.io import (load_physio, save_physio, load_history, save_history)
from peakdet.operations import (delete_peaks, edit_physio, filter_physio,
                                interpolate_physio, peakfind_physio,
                                plot_physio, reject_peaks)
from peakdet.physio import (Physio)

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions


# -*- coding: utf-8 -*-
"""Functions and class for performing interactive editing of physiological data."""

import functools
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.widgets import SpanSelector
from peakdet import operations, utils


class _PhysioEditor():
    """
    Class for editing physiological data.

    Parameters
    ----------
    data : Physio_like
        Physiological data to be edited
    """

    def __init__(self, data):
        # save reference to data and generate "time" for interpretable X-axis
        self.data = utils.check_physio(data, copy=True)
        fs = 1 if data.fs is None else data.fs
        self.time = np.arange(0, len(data.data) / fs, 1 / fs)
        # Read if there is support data
        self.suppdata = data.suppdata

        # we need to create these variables in case someone doesn't "quit"
        # the plot appropriately (i.e., clicks X instead of pressing ctrl+q)
        self.deleted, self.rejected, self.included = set(), set(), set()

        # make main plot objects depending on supplementary data
        if self.suppdata is None:
            self.fig, self._ax = plt.subplots(nrows=1, ncols=1,
                                              tight_layout=True, sharex=True)
        else:
            self.fig, self._ax = plt.subplots(nrows=2, ncols=1,
                                              tight_layout=True, sharex=True,
                                              gridspec_kw={'height_ratios': [3, 2]})

        self.fig.canvas.mpl_connect('scroll_event', self.on_wheel)
        self.fig.canvas.mpl_connect('key_press_event', self.on_key)

        # Set axis handler
        self.ax = self._ax if self.suppdata is None else self._ax[0]

        # three selectors for:
        #    1. rejection (central mouse),
        #    2. addition (right mouse), and
        #    3. deletion (left mouse)
        delete = functools.partial(self.on_edit, method='delete')
        reject = functools.partial(self.on_edit, method='reject')
        insert = functools.partial(self.on_edit, method='insert')
        self.span2 = SpanSelector(self.ax, delete, 'horizontal',
                                  button=1, useblit=True,
                                  rectprops=dict(facecolor='red', alpha=0.3))
        self.span1 = SpanSelector(self.ax, reject, 'horizontal',
                                  button=2, useblit=True,
                                  rectprops=dict(facecolor='blue', alpha=0.3))
        self.span3 = SpanSelector(self.ax, insert, 'horizontal',
                                  button=3, useblit=True,
                                  rectprops=dict(facecolor='green', alpha=0.3))

        self.plot_signals(False)

    def plot_signals(self, plot=True):
        """Clear axes and plots data / peaks / troughs."""
        # don't reset x-/y-axis zooms on replot
        if plot:
            xlim, ylim = self.ax.get_xlim(), self.ax.get_ylim()
        else:
            xlim, ylim = (-5, None), (None, None)

        # clear old data + redraw, retaining x-/y-axis zooms
        self.ax.clear()
        self.ax.plot(self.time, self.data, 'b',
                     self.time[self.data.peaks],
                     self.data[self.data.peaks], '.r',
                     self.time[self.data.troughs],
                     self.data[self.data.troughs], '.g')

        if self.suppdata is not None:
            self._ax[1].plot(self.time, self.suppdata, 'k', linewidth=0.7)
            self._ax[1].set_ylim(-.5, .5)

        self.ax.set(xlim=xlim, ylim=ylim, yticklabels='')
        self.fig.canvas.draw()

    def on_wheel(self, event):
        """Move axis on wheel scroll."""
        (xlo, xhi), move = self.ax.get_xlim(), event.step * -10
        self.ax.set_xlim(xlo + move, xhi + move)
        self.fig.canvas.draw()

    def quit(self):
        """Quit editor."""
        plt.close(self.fig)

    def on_key(self, event):
        """Undo last span select or quits peak editor."""
        # accept both control or Mac command key as selector
        if event.key in ['ctrl+z', 'super+d']:
            self.undo()
        elif event.key in ['ctrl+q', 'super+d']:
            self.quit()

    def on_edit(self, xmin, xmax, *, method):
        """
        Edit peaks by rejection, deletion, or insert.

        Removes specified peaks by either rejection / deletion, OR
        Include one peak by finding the max in the selection.

        method accepts 'insert', 'reject', 'delete'
        """
        if method not in ['insert', 'reject', 'delete']:
            raise ValueError(f'Action "{method}" not supported.')

        tmin, tmax = np.searchsorted(self.time, (xmin, xmax))
        pmin, pmax = np.searchsorted(self.data.peaks, (tmin, tmax))

        if method == 'insert':
            tmp = np.argmax(self.data.data[tmin:tmax]) if tmin != tmax else 0
            newpeak = tmin + tmp
            if newpeak == tmin:
                self.plot_signals()
                return
        else:
            bad = np.arange(pmin, pmax, dtype=int)
            if len(bad) == 0:
                self.plot_signals()
                return

        if method == 'reject':
            rej, fcn = self.rejected, operations.reject_peaks
        elif method == 'delete':
            rej, fcn = self.deleted, operations.delete_peaks

        # store edits in local history & call function
        if method == 'insert':
            self.included.add(newpeak)
            self.data = operations.add_peaks(self.data, newpeak)
        else:
            rej.update(self.data.peaks[bad].tolist())
            self.data = fcn(self.data, self.data.peaks[bad])

        self.plot_signals()

    def undo(self):
        """Reset last span select peak removal."""
        # check if last history entry was a manual reject / delete
        relevant = ['reject_peaks', 'delete_peaks', 'add_peaks']
        if self.data._history[-1][0] not in relevant:
            return

        # pop off last edit and delete
        func, peaks = self.data._history.pop()

        if func == 'reject_peaks':
            self.data._metadata['reject'] = np.setdiff1d(
                self.data._metadata['reject'], peaks['remove']
            )
            self.rejected.difference_update(peaks['remove'])
        elif func == 'delete_peaks':
            self.data._metadata['peaks'] = np.insert(
                self.data._metadata['peaks'],
                np.searchsorted(self.data._metadata['peaks'], peaks['remove']),
                peaks['remove']
            )
            self.deleted.difference_update(peaks['remove'])
        elif func == 'add_peaks':
            self.data._metadata['peaks'] = np.delete(
                self.data._metadata['peaks'],
                np.searchsorted(self.data._metadata['peaks'], peaks['add']),
            )
            self.included.remove(peaks['add'])
        self.data._metadata['troughs'] = utils.check_troughs(self.data,
                                                             self.data.peaks,
                                                             self.data.troughs)
        self.plot_signals()



# -*- coding: utf-8 -*-

import pytest
from peakdet import external
from peakdet.tests import utils as testutils

DATA = testutils.get_test_data_path('rtpeaks.csv')


def test_load_rtpeaks():
    for channel in [1, 2, 9]:
        with pytest.warns(UserWarning):
            hist = dict(fname=DATA, channel=channel, fs=1000.)
            phys = external.load_rtpeaks(DATA, channel=channel, fs=1000.)
            assert phys.history == [('load_rtpeaks', hist)]
            assert phys.fs == 1000.
            with pytest.raises(ValueError):
                external.load_rtpeaks(testutils.get_test_data_path('ECG.csv'),
                                      channel=channel, fs=1000.)



# -*- coding: utf-8 -*-

from peakdet import analytics
from peakdet.tests.utils import get_peak_data

ATTRS = [
    'rrtime', 'rrint', 'avgnn', 'sdnn', 'rmssd', 'sdsd', 'nn50', 'pnn50',
    'nn20', 'pnn20', 'hf', 'hf_log', 'lf', 'lf_log', 'vlf', 'vlf_log',
    'lftohf', 'hf_peak', 'lf_peak'
]


def test_HRV():
    peaks = get_peak_data()
    hrv = analytics.HRV(peaks)
    for attr in ATTRS:
        assert hasattr(hrv, attr)
